@@ -20,7 +20,9 @@
 #include "config.h"
 
 #include "ephy-search-provider.h"
+#include "ephy-embed-shell.h"
 #include "ephy-file-helpers.h"
+#include "ephy-private.h"
 
 gint main (gint argc, gchar** argv)
 {
@@ -35,6 +37,8 @@ gint main (gint argc, gchar** argv)
     return 1;
   }
 
+  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_SEARCH_PROVIDER);
+
   search_provider = ephy_search_provider_new ();
   status = g_application_run (G_APPLICATION (search_provider), argc, argv);
   g_object_unref (search_provider);
