@@ -1091,7 +1091,7 @@ window_cmd_edit_find (GtkAction *action,
 	EphyFindToolbar *toolbar;
 	
 	toolbar = EPHY_FIND_TOOLBAR (ephy_window_get_current_find_toolbar (window));
-	ephy_find_toolbar_open (toolbar, FALSE, FALSE);
+	ephy_find_toolbar_toggle_state (toolbar);
 }
 
 void
